﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.AI;

public class EnemyController : MonoBehaviour {

	public float speed, jump, dashSpeed;
	public Transform[] points;
	public GameObject player;
	public AudioClip jumpSound, dashSound;

	private PlayerController playerCtrl;
	private Rigidbody rigidBody;
	private NavMeshAgent agent;
	private Vector3 movement, jumpDirection, direction, distance;
	private int destination = 0, hard;
	private bool onGround, isStunned, isChasing, isJumping, isDashing;
	private float lastStun, lastDash;
	private GameObject gameObj;
	private AudioSource audiSrc;
	private Animator anim;

	void Start () {
		rigidBody = GetComponent<Rigidbody> ();
		agent = GetComponent<NavMeshAgent> ();
		agent.autoBraking = false;
		distance = player.transform.position - rigidBody.transform.position;
		gameObj = GameObject.FindGameObjectWithTag ("GameController");
		playerCtrl = player.GetComponent<PlayerController> ();
		audiSrc = GetComponent<AudioSource> ();
		anim = GetComponent<Animator> ();
		hard = PlayerPrefs.GetInt ("Difficulty");
	}

	void Update () {
		distance = player.transform.position - rigidBody.transform.position;
		float moveH = distance.normalized.x;
		float moveV = distance.normalized.z;
		movement = new Vector3 (moveH, 0.0f, moveV);
		direction = rigidBody.velocity.normalized;

		isChasing = (distance.magnitude < 30) && playerCtrl.inDen;

		agent.enabled = !isChasing;

		if (!isChasing && !agent.pathPending && agent.remainingDistance < 0.5f) {
			agent.destination = points [destination].position;
			destination = (destination + 1) % points.Length;
		}

		AnimatorScript aScript = gameObj.GetComponent<AnimatorScript> ();

		anim.SetBool (aScript.chaseHash, isChasing);

	}

	void FixedUpdate () {
		if (Mathf.Abs (rigidBody.velocity.magnitude) > 15) {
			rigidBody.velocity = new Vector3(direction.x * 15, rigidBody.velocity.y, direction.z * 15) ;
		}
		onGround = Physics.Raycast (transform.position, jumpDirection * -1, 0.75f);
		if (isChasing) {
			Move ();
			if (hard > 0) {
				Jump ();
				Dash ();
			}
		}
	}

	void Move() {
		if ((playerCtrl.onNest && distance.magnitude < 7) || (distance.y > 8)|| isStunned ) {
			rigidBody.velocity = new Vector3 (0.0f, rigidBody.velocity.y, 0.0f) * Time.fixedDeltaTime;
			rigidBody.transform.LookAt (playerCtrl.transform.position, Vector3.zero * Time.fixedTime);
		} else {
			rigidBody.AddForce (movement * speed);
		} if (Time.time - lastStun > 3) {
			isStunned = false;
		}
	}

	void Jump() {
		if ((distance.y > 2) && (distance.y <= 8) &&onGround && !isJumping && !isStunned) {
			rigidBody.AddForce (jumpDirection * jump, ForceMode.Impulse);
			isJumping = true;
			audiSrc.clip = jumpSound;
			audiSrc.Play ();
		}
	}

	void Dash() {
		if ((distance.magnitude < 8) && (distance.y <= 2) && onGround && !isDashing && !isStunned) { 
			rigidBody.AddForce (direction * dashSpeed, ForceMode.Impulse);
			if (direction.magnitude == 0) {
				rigidBody.AddForce (Vector3.forward * dashSpeed, ForceMode.Impulse);
			}
			audiSrc.clip = dashSound;
			audiSrc.Play ();
			isDashing = true;
			lastDash = Time.time;
		} if (Time.time - lastDash > 1) {
			isDashing = false;
		} 
	}

	private void OnCollisionStay(Collision collision) {
		Vector3 temp = jumpDirection;
		if (Vector3.Angle (Vector3.up, collision.contacts [0].normal) < 90 && !collision.gameObject.CompareTag ("Enemy")) {
			jumpDirection = collision.contacts [0].normal;
		} else {
			jumpDirection = temp;
		}
	}

	private void OnCollisionEnter(Collision collision) {
		if (collision.gameObject.CompareTag ("Player") && playerCtrl.isDashing) {
			isStunned = true;
			lastStun = Time.time;
		}
		isJumping = false;
	}
}
