﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.SceneManagement;

public class GameController : MonoBehaviour {
	public Text currentLeavesText, leavesLeftText, scoreText, foxScoreText, infoText, finalScoreText, finalFoxText, outcomeText;
	public List<GameObject> pickups;
	public GameObject player, enemy, hud, pauseMenu, endMenu;
	public SaveController saver;
	public int currentLeaves;
	public GameObject nest;
	public Material playerMaterial, enemyMaterial;
	public Texture playerTexture, enemyTexture;
	public bool pause;

	private int leavesLeft, score, foxScore;
	private int numberOfPickups;
	private bool finish, lift;
	private float liftTime, volume;

	void Awake() {
		playerMaterial.SetTexture ("_MainTex", playerTexture);
		enemyMaterial.SetTexture ("_MainTex", enemyTexture);
	}

	void Start () {

		numberOfPickups = pickups.Count;
		currentLeaves = 0;
		leavesLeft = numberOfPickups;
		score = 0;
		foxScore = 0;
		SetCountTexts ();

		volume = PlayerPrefs.GetFloat ("Volume");
		AudioListener.volume = volume;
		saver = GameObject.FindGameObjectWithTag ("SaveController").GetComponent<SaveController> ();
	}

	void Update () {
		if (!finish) {
			SetCountTexts ();
		}
		if (Time.time - liftTime > 8) {
			lift = false;
			infoText.text = "";
		}

		if (Input.GetKeyDown(KeyCode.P) && !finish) {
			if (!pause) {
				pauseMenu.SetActive (true);
				hud.SetActive (false);
				Time.timeScale = 0;
				pause = true;
				GameObject.Find("Button sound").GetComponent<AudioSource> ().Play ();
				gameObject.GetComponent<AudioSource> ().Pause ();
			} else {
				pauseMenu.SetActive (false);
				hud.SetActive (true);
				Time.timeScale = 1;
				pause = false;
				GameObject.Find("Button sound").GetComponent<AudioSource> ().Play ();
				gameObject.GetComponent<AudioSource> ().UnPause ();
			}
		}
	}

	public void SetCountTexts() {
		currentLeavesText.text = "Current leaves: " + currentLeaves.ToString ();
		leavesLeftText.text = "Leaves left: " + leavesLeft.ToString ();
		scoreText.text = "Score: " + score.ToString ();
		foxScoreText.text = "Fox's score: " + foxScore.ToString ();
		if (leavesLeft + currentLeaves == 0) {
			finish = true;
			endMenu.SetActive (true);
			hud.SetActive (false);
			Time.timeScale = 0;
			gameObject.GetComponent<AudioSource> ().Stop ();
			SetResultTexts ();
		}
		if (lift) {
			infoText.text = "The lift on the back of the tower is now on!";
		}
	}

	public void SetResultTexts() {
		finalScoreText.text = "Your Score: " + score;
		finalFoxText.text = "Fox's Score: " + foxScore;
		if (score > foxScore) {
			saver.unlock = true;
			outcomeText.text = "Outcome: You win!";
		} else if (score < foxScore) {
			outcomeText.text = "Outcome: You lose!";
		} else {
			outcomeText.text = "Outcome: Draw!";
		}
	}

	public void incCount() {
		currentLeaves++;
		leavesLeft--;
	}

	public void bankLeaves() {
		score += (int)(10 * (currentLeaves + (currentLeaves * ((float)currentLeaves / 10))));
		currentLeaves = 0;
		GameObject.FindGameObjectWithTag ("Nest").GetComponent<AudioSource> ().Play ();
	}

	public void foxBankLeaves(int i) {
		foxScore += (int)(20 * (i + (i * ((float)i / 20))));
	}

	public void hit(GameObject obj) {
		if (currentLeaves >= 5) {
			currentLeaves -= 5;
			if (obj.CompareTag ("Enemy")) {
				foxBankLeaves (5);
			}
		} else {
			if (obj.CompareTag ("Enemy")) {
				foxBankLeaves(currentLeaves);
			}
			currentLeaves = 0;
		}
	}

	public void activateLift() {
		GameObject.FindGameObjectWithTag ("Lift").GetComponent<LiftController> ().enabled = true;
		lift = true;
		liftTime = Time.time;
	}
}
