﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class PlayerController : MonoBehaviour {

	public float speed, jump, dashSpeed;
	public bool onNest, isDashing, inDen;
	public int cam;
	public AudioClip jumpSound, dashSound, hitSound, leafSound;
	public Vector3 direction;

	private Rigidbody rigidBody;
	private Vector3 movementZ, movementX, jumpDirection;
	private bool onGround, mouseHeldL, mouseHeldR;
	private float lastDash;
	private GameObject gameObj;
	private GameController gameCtrl;
	private AudioSource audioSrc;
	private Animator anim;

	void Start () {
		rigidBody = GetComponent<Rigidbody> ();
		gameObj = GameObject.FindGameObjectWithTag ("GameController");
		gameCtrl = gameObj.GetComponent<GameController> ();
		lastDash = -5;
		cam = 1;
		direction = new Vector3(0, 0, 1);
		audioSrc = GetComponent<AudioSource> ();
		anim = GetComponent<Animator> ();
		isDashing = false;
		inDen = false;
	}

	void Update () {
		float moveH = Input.GetAxis ("Horizontal");
		float moveV = Input.GetAxis ("Vertical");

		Vector3 camDirection = Camera.main.transform.forward;
		movementZ = new Vector3(Camera.main.transform.forward.x, 0.0f, Camera.main.transform.forward.z) * moveV;
		movementX = new Vector3(Camera.main.transform.right.x, 0.0f, Camera.main.transform.right.z) * moveH;
			
		if (rigidBody.velocity.magnitude > 0) {
			direction = rigidBody.velocity.normalized;
		}
	}

	void FixedUpdate () {
		if (Mathf.Abs (rigidBody.velocity.magnitude) > 15) {
			rigidBody.velocity = new Vector3(direction.x * 15, rigidBody.velocity.y, direction.z * 15) ;
		}
		onGround = Physics.Raycast (transform.position, Vector3.down, 0.75f);
		Move ();
		Jump ();
		Dash ();
	}

	void LateUpdate() {
		AnimatorScript aScript = gameObj.GetComponent<AnimatorScript> ();

		anim.SetBool (aScript.dashHash, isDashing);
	}

	void Move() {
		rigidBody.AddForce ((movementZ + movementX) * speed);
	}


	void Jump() {
		if (Input.GetButton("Jump") && onGround && !mouseHeldL) {
			rigidBody.AddForce (jumpDirection * jump, ForceMode.Impulse);
			mouseHeldL = true;
			audioSrc.clip = jumpSound;
			audioSrc.Play ();
		} if (Input.GetButtonUp ("Jump")) {
			mouseHeldL = false;
		}
	}

	void Dash() {
		if (Input.GetButton("Fire1") && onGround && !mouseHeldR) { 
			rigidBody.AddForce ((movementZ + movementX) * dashSpeed, ForceMode.Impulse);
			if ((movementZ + movementX).magnitude == 0) {
				rigidBody.AddForce (direction * dashSpeed, ForceMode.Impulse);
			}
			else if (direction.magnitude == 0) {
				rigidBody.AddForce (Vector3.forward * dashSpeed, ForceMode.Impulse);
			}
			audioSrc.clip = dashSound;
			audioSrc.Play ();
			isDashing = true;
			mouseHeldR = true;
			lastDash = Time.time;
		} if (Time.time - lastDash > 1) {
			isDashing = false;
		} if (!Input.GetButton("Fire1") && Time.time - lastDash > 1) {
			mouseHeldR = false;
		}
	}

	private void OnCollisionStay(Collision collision) {
		Vector3 temp = jumpDirection;
		if (Vector3.Angle (Vector3.up, collision.contacts [0].normal) < 90 && !collision.gameObject.CompareTag ("Enemy")) {
			jumpDirection = collision.contacts [0].normal;
		} else {
			jumpDirection = temp;
		}
	}
			

	private void OnTriggerEnter(Collider other) {
		if (other.gameObject.CompareTag ("Pickup")) {
			audioSrc.clip = leafSound;
			audioSrc.Play ();
			DestroyObject (other.gameObject);
			gameCtrl.incCount ();
			gameCtrl.SetCountTexts ();
			gameCtrl.pickups.Remove (other.gameObject);
		}
		if (other.gameObject.CompareTag ("Pit")) {
			transform.position = new Vector3 (gameCtrl.nest.transform.position.x, gameCtrl.nest.transform.position.y + 2, gameCtrl.nest.transform.position.z);
			gameCtrl.hit (other.gameObject);
			audioSrc.clip = hitSound;
			audioSrc.Play ();
		}

		if (other.gameObject.CompareTag("Key")) {
			audioSrc.clip = leafSound;
			audioSrc.Play ();
			DestroyObject (other.gameObject);
			gameCtrl.activateLift ();
		}

		if (other.gameObject.CompareTag("Den")) {
			inDen = true;
		}
	}

	private void OnTriggerExit(Collider other) {
		if (other.gameObject.CompareTag("Den")) {
			inDen = false;
		}
	}

	private void OnCollisionEnter(Collision collision) {
		if (collision.gameObject.CompareTag ("Nest")) {
			gameCtrl.nest = collision.gameObject;
			gameCtrl.bankLeaves ();
			onNest = true;
		}


		if (collision.gameObject.CompareTag ("Enemy") && !isDashing) {
			rigidBody.AddForce (collision.contacts [0].point, ForceMode.Impulse);
			collision.gameObject.GetComponent<Rigidbody>().AddForce (collision.contacts [0].point * -0.25f, ForceMode.Impulse);
			gameCtrl.hit (collision.gameObject);
			audioSrc.clip = hitSound;
			audioSrc.Play ();
		}

		if (collision.gameObject.CompareTag("Mud")) {
			collision.gameObject.GetComponent<AudioSource> ().Play();
		}
	}

	private void OnCollisionExit(Collision collision) {
		if (collision.gameObject.CompareTag ("Nest")) {
			onNest = false;
		}
	}
}
